@@ -1,7 +1,7 @@
 #ifndef _SUSPEND_USERUI_H_
 #define _SUSPEND_USERUI_H_
 
-#define SUSPEND_USERUI_INTERFACE_VERSION 6
+#define SUSPEND_USERUI_INTERFACE_VERSION 7
 
 enum {
 	USERUI_MSG_BASE = 0x10,
@@ -16,6 +16,8 @@ enum {
 	USERUI_MSG_NOFREEZE_ME = 0x16,
 	USERUI_MSG_SPACE = 0x18,
 	USERUI_MSG_GET_DEBUGGING = 0x19,
+	USERUI_MSG_GET_POWERDOWN_METHOD = 0x1A,
+	USERUI_MSG_SET_POWERDOWN_METHOD = 0x1B,
 
 	/* Kernel -> Userspace */
 	USERUI_MSG_MESSAGE = 0x21,
