@@ -5,7 +5,7 @@
 #include <sys/types.h>
 #include "suspend_userui.h"
 
-#define USERUI_VERSION "0.6.4"
+#define USERUI_VERSION "0.7.0"
 
 struct userui_ops {
 	char *name;
@@ -124,8 +124,11 @@ static __inline__ int generic_fls(int x)
 /*
  * Work around random glibc bugs where getpid() caches an invalid pid.
  */
+#ifdef _syscall0
 #define __NR_xgetpid __NR_getpid
 static inline _syscall0(pid_t, xgetpid);
-
+#else
+static inline pid_t xgetpid(void) { return syscall(__NR_getpid); }
+#endif
 
 #endif /* _USERUI_H_ */
