@@ -49,6 +49,7 @@ static int need_cleanup = 0;
 static int safe_to_exit = 1;
 static volatile int need_loglevel_change = 0;
 static int debugging_enabled = 0;
+static int powerdown_method = 0;
 
 char software_suspend_version[32];
 int can_use_escape = 0;
@@ -169,6 +170,39 @@ static void toggle_debug_state(key) {
 	userui_ops->message(0, SUSPEND_UI_MSG, 1, message);
 }
 
+static void toggle_poweroff(void) {
+	char message[80];
+	static int other_powerdown_method = 0;
+
+	/* 
+	 * Toggle between 3 and the original powerdown_method. If the
+	 * original method is 3, toggle between that and zero (safest).
+	 *
+	 * This may have no effect in the end, though, because rebooting
+	 * always trumps powering down.
+	 */
+	if (powerdown_method != 3) {
+		other_powerdown_method = powerdown_method;
+		powerdown_method = 3;
+
+		if ((suspend_action & (1 << SUSPEND_REBOOT)))
+			sprintf(message, "Would suspend to ram but rebooting is enabled.");
+		else
+			sprintf(message, "Suspend to ram on completion.");
+	} else {
+		powerdown_method = other_powerdown_method;
+		if ((suspend_action & (1 << SUSPEND_REBOOT)))
+			sprintf(message, "Would power off but rebooting is enabled.");
+		else
+			sprintf(message, "Power off on completion.");
+	}
+
+	send_message(USERUI_MSG_SET_POWERDOWN_METHOD, (int*)&powerdown_method,
+			sizeof(powerdown_method));
+
+	userui_ops->message(0, SUSPEND_UI_MSG, 1, message);
+}
+
 static void toggle_pause() {
 	if (!debugging_enabled)
 		return;
@@ -234,6 +268,9 @@ int common_keypress_handler(int key) {
 		case 0x13: /* R */
 			toggle_reboot();
 			break;
+		case 0x18: /* O */
+			toggle_poweroff();
+			break;
 		case 0x19: /* P */
 			toggle_pause();
 			break;
@@ -389,6 +426,10 @@ static void get_info() {
 		bail_err("send_message");
 	}
 
+	if (!send_message(USERUI_MSG_GET_POWERDOWN_METHOD, NULL, 0)) {
+		bail_err("send_message");
+	}
+	
 	/* We'll get the replies in our message loop */
 
 	get_console_loglevel();
@@ -473,7 +514,7 @@ static void restore_console() {
 static void sig_hand(int sig) {
 	restore_console();
 
-	printf("userui: Ack! SIG %d\n", *(int*)sig);
+	printf("userui: Ack! SIG %d\n", sig);
 
 	if (test_run)
 		exit(1);
@@ -724,35 +765,15 @@ static void unblank_screen() {
 }
 
 static void message_loop() {
-	static char buf1[4096], buf2[4096];
-	struct nlmsghdr *nlh, *nlh2;
+	static char buf1[4096];
+	struct nlmsghdr *nlh;
 
 	while (1) {
 		struct userui_msg_params *msg;
-		int missed_progress_events = 0;
 
 		if (!(nlh = fetch_message(buf1, sizeof(buf1), 0)))
 			return; /* EOF */
 
-		while (nlh->nlmsg_type == USERUI_MSG_PROGRESS ||
-				nlh->nlmsg_type == USERUI_MSG_MESSAGE) {
-			nlh2 = fetch_message(buf2, sizeof(buf2), 1);
-			if (nlh2 == NULL)
-				break;
-
-			if (nlh2->nlmsg_type == USERUI_MSG_PROGRESS) {
-				if (nlh->nlmsg_type == USERUI_MSG_MESSAGE) {
-					/* Simply ignore the progress message */
-					continue;
-				}
-				/* Otherwise, ignore the message in nlh and keep on reading */
-				memcpy(buf1, buf2, sizeof(buf2));
-				missed_progress_events++;
-				if (missed_progress_events == 20)
-					break;
-			}
-		}
-
 		msg = NLMSG_DATA(nlh);
 
 		switch (nlh->nlmsg_type) {
@@ -771,8 +792,12 @@ static void message_loop() {
 			case USERUI_MSG_IS_DEBUGGING:
 				debugging_enabled = *(int *)NLMSG_DATA(nlh);
 				break;
+			case USERUI_MSG_GET_POWERDOWN_METHOD:
+				powerdown_method = *(int *)NLMSG_DATA(nlh);
+				break;
 			case USERUI_MSG_CLEANUP:
 				userui_ops->cleanup();
+				send_message(USERUI_MSG_CLEANUP, NULL, 0);
 				close(nlsock);
 				exit(0);
 			case USERUI_MSG_REDRAW:
@@ -875,8 +900,7 @@ int main(int argc, char **argv) {
 	enforce_lifesavers();
 
 	if (test_run) {
-	test_run = 0;
-	safe_to_exit = 0;
+		safe_to_exit = 0;
 
 		do_test_run();
 		return 0;
